import {useEffect, useState} from 'react';
import * as Sentry from '@sentry/react';

import type {Docs} from 'sentry/components/onboarding/gettingStartedDoc/types';
import {
  feedbackOnboardingPlatforms,
  replayPlatforms,
} from 'sentry/data/platformCategories';
import type {Organization, PlatformIntegration, Project} from 'sentry/types';
import {getPlatformPath} from 'sentry/utils/gettingStartedDocs/getPlatformPath';
import {useProjectKeys} from 'sentry/utils/useProjectKeys';

type Props = {
  orgSlug: Organization['slug'];
  platform: PlatformIntegration;
  productType?: 'feedback' | 'replay';
  projSlug?: Project['slug'];
};

export function useLoadGettingStarted({
  platform,
  productType,
  orgSlug,
  projSlug,
}: Props): {
  cdn: string | undefined;
  docs: Docs<any> | null;
  dsn: string | undefined;
  isError: boolean;
  isLoading: boolean;
  refetch: () => void;
} {
  const [module, setModule] = useState<undefined | 'none' | {default: Docs<any>}>(
    undefined
  );

  const projectKeys = useProjectKeys({orgSlug, projSlug});

  const platformPath = getPlatformPath(platform);

  useEffect(() => {
    async function getGettingStartedDoc() {
      if (
        (productType === 'replay' && !replayPlatforms.includes(platform.id)) ||
        (productType === 'feedback' && !feedbackOnboardingPlatforms.includes(platform.id))
      ) {
        setModule('none');
        return;
      }

      try {
        const mod = await import(
          /* webpackExclude: /.spec/ */
          `sentry/gettingStartedDocs/${platformPath}`
        );
        setModule(mod);
      } catch (err) {
        setModule(undefined);
        Sentry.captureException(err);
      }
    }

    getGettingStartedDoc();

    return () => {
      setModule(undefined);
    };
  }, [platformPath, platform.id, productType]);

  return {
    refetch: projectKeys.refetch,
    isLoading: projectKeys.isLoading || module === undefined,
    isError: projectKeys.isError,
    docs: module === 'none' ? null : module?.default ?? null,
    dsn: projectKeys.data?.[0]?.dsn.public,
    cdn: projectKeys.data?.[0]?.dsn.cdn,
  };
}
